<?php
/**
 * Custom filters for this theme
 * @package Twenty8teen
 */

/**
 * Adds custom classes to the array of body classes.
 *
 * @param array $classes Classes for the body element.
 * @return array
 */
function twenty8teen_body_classes( $classes ) {
	if ( is_singular() ) {
		// Adds a class of the post slug to singular pages.
		$post = get_post();
		$classes[] = 'name-' . esc_attr( $post->post_name );
	}
	else {
		// Adds a class of hfeed to non-singular pages.
		$classes[] = 'hfeed';
	}

	$defaults = twenty8teen_default_booleans();
	if ( get_theme_mod( 'show_vignette', 	$defaults['show_vignette'] ) ) {
		$classes[] = 'vignette';
	}
	if ( get_theme_mod( 'show_header_imagebehind', 	$defaults['show_header_imagebehind'] ) ) {
		$classes[] = 'header-behind';
	}
	if ( is_active_sidebar( 'side-widget-area' ) ) {
		$classes[] = 'has-sidebar';
		if ( get_theme_mod( 'switch_sidebar', 	$defaults['switch_sidebar'] ) ) {
			$classes[] = 'sidebar-leading';
		}
	}

	return $classes;
}
add_filter( 'body_class', 'twenty8teen_body_classes' );

/**
 * Add entry class for all post types.
 * Remove hentry class from the array of post classes, for Pages.
 * Add the cards class if set.
 */
function twenty8teen_post_classes( $classes, $class, $post_id ) {
	$classes[] = 'entry';
	if ( is_page() ) {
		if ( $i = array_search( 'hentry', $classes ) ) {
			unset( $classes[$i] );
		}
	}

	$defaults = twenty8teen_default_booleans();
	if ( get_theme_mod( 'show_as_cards', $defaults['show_as_cards'] ) ) {
		$classes[] = 'cards';
	}

	return $classes;
}
add_filter( 'post_class', 'twenty8teen_post_classes', 20, 3 );


/**
 * Adds custom classes to the array of attributes.
 *
 * @see twenty8teen_attributes
 * @return array
 */
function twenty8teen_add_page_classes( $attrs, $tag ) {
	if ( is_page() && isset( $attrs['class'] ) ) {
		$class = is_array( $attrs['class'] ) ? $attrs['class'] : explode( ' ', $attrs['class'] );
		if ( in_array( 'entry-header', $class ) ) {
			$class[] = 'page-header';
		}
		if ( in_array( 'entry-title', $class ) ) {
			$class[] = 'page-title';
		}
		if ( in_array( 'entry-content', $class ) ) {
			$class[] = 'page-content';
		}
		if ( in_array( 'entry-footer', $class ) ) {
			$class[] = 'page-footer';
		}
		$attrs['class'] = $class;
	}
	return $attrs;
}
add_filter( 'twenty8teen_attributes', 'twenty8teen_add_page_classes', 10, 2 );

/**
 * Adds user classes to the array of area class choices.
 */
function twenty8teen_add_user_classes( $choices ) {
	$user_classes = get_theme_mod( 'user_classes', '' );
	if ( $user_classes ) {
		$user_classes = explode( ' ', $user_classes );
		$user_classes = array_map( 'esc_attr', $user_classes );
		$display = array_map( 'ucfirst', $user_classes );
		$choices = $choices + array_combine( $user_classes, $display );
	}
	return $choices;
}
add_filter( 'twenty8teen_area_class_choices', 'twenty8teen_add_user_classes' );

/**
 * For pagination and navigation, adds classes chosen in widgets. Used in template files.
 * 'navigation_markup_template' filter
 */
function twenty8teen_nav_add_widget_classes( $template, $class ) {
	$add = twenty8teen_widget_get_classes();
	if ( $add ) {
		$template = str_replace( '%1$s', $add . ' %1$s', $template );
	}
	return $template;
}

/**
 * For custom logo, adds classes chosen in widgets. Used in logo template file.
 */
function twenty8teen_logo_add_widget_classes( $attr, $attachment, $size ) {
	$add = twenty8teen_widget_get_classes();
	if ( $add ) {
		$attr['class'] .= ' ' . $add;
	}
	return $attr;
}

/**
 * For attachment pages, output the caption as content.
 */
function twenty8teen_attachment( $p ) {
	$post = get_post();
	if ( wp_attachment_is( 'video', $post ) || wp_attachment_is( 'audio', $post ) ) {
		// Already handled.
	}
	else { // non-video and non-audio - use large size or icon
		$p = '<p class="attachment">';
		$p .= wp_get_attachment_link( $post->ID, 'large', false, true);
		$p .= '</p>';
	}
	$p .= $post->post_excerpt ? ( '<p>'. esc_html( $post->post_excerpt ) . '</p>' ) : '';
	return $p;
}
add_filter('prepend_attachment', 'twenty8teen_attachment');

/**
 * Generate the style rules for the editor to look like front end.
 */
function twenty8teen_editor_dynamic_rules( $mce_init ) {
	$css = twenty8teen_dynamic_rules( true ) . ' ';
	$css = str_replace( array( '"', "\n" ), array( '\"',' ' ) , $css );
	if ( isset( $mce_init['content_style'] ) ) {
		$mce_init['content_style'] .= ' ' . $css;
	} else {
		$mce_init['content_style'] = $css;
	}
	return $mce_init;
}
add_filter('tiny_mce_before_init', 'twenty8teen_editor_dynamic_rules');

/**
 * Change the post excerpt length to user's choice.
 */
function twenty8teen_excerpt_length( $length ) {
	if ( ! is_admin() ) {
		$default = twenty8teen_default_sizes();
		$length = absint( get_theme_mod( 'excerpt_length', $default['excerpt_length'] ) );
	}
	return $length;
}
add_filter( 'excerpt_length', 'twenty8teen_excerpt_length' );

/**
 * Add a fragment identifier (to the content) to paginated links.
 */
function twenty8teen_link_pages_link( $link, $i ) {
	if ( $i > 1 && preg_match( '/href="([^"]*)"/', $link, $matches ) ) {
		if ( false === strpos( $matches[1], '#' ) ) {
			$link = str_replace( $matches[1], $matches[1] . '#content', $link );
		}
	}
	return $link;
}
add_filter( 'wp_link_pages_link', 'twenty8teen_link_pages_link', 10, 2 );

/**
 * Supply default value for sidebar widgets.
 */
function twenty8teen_default_widgets( $default, $option, $passed_default ) {
	$default = array(
		'header-widget-area' => array( 'twenty8teen-template-part-2',
			'twenty8teen-template-part-3' ),
		'content-widget-area' => array( 'twenty8teen-loop-part-2',
			'twenty8teen-template-part-4' ),
		'side-widget-area' => array(),
		'footer-widget-area' => array( 'twenty8teen-template-part-5' ),
	);
	if ( 'theme_mods_twenty8teen' === $option ) {
		$default = array(
			'sidebars_widgets' => array( 'time' => time(), 'data' => $default ),
		);
	}
	return $default;
}
// Will only affect a new site installation.
add_filter( 'default_option_sidebars_widgets', 'twenty8teen_default_widgets', 1, 3 );
// Will only affect first-time theme installation.
add_filter( 'default_option_theme_mods_twenty8teen', 'twenty8teen_default_widgets', 1, 3 );

/**
 * Supply default value for template-parts widgets.
 */
function twenty8teen_default_widget_template_parts( $default, $option, $passed_default ) {
	return array(
		2 => array( 'title' => 'Site Branding', 'part' => 'site-branding' ),
		3 => array( 'title' => 'Main Nav', 'part' => 'main-nav' ),
		4 => array( 'title' => 'Posts Pagination', 'part' => 'posts-pagination' ),
		5 => array( 'title' => 'Site Info', 'part' => 'site-info' ),
		'_multiwidget' => 1,
	);
}
add_filter( 'default_option_widget_twenty8teen-template-part',
	'twenty8teen_default_widget_template_parts', 1, 3 );

/**
 * Supply default value for loop-parts widgets.
 */
function twenty8teen_default_widget_loop_parts( $default, $option, $passed_default ) {
	return array(
		2 => array(
			'title' => 'Entry, Post Navigation, Comments',
			'part' => array(
				'entry',
				'post-navigation',
				'comments',
			),
			'align' => array( '', '', '' ),
			'class' => array(),
		),
		'_multiwidget' => 1,
	);
}
add_filter( 'default_option_widget_twenty8teen-loop-part',
	'twenty8teen_default_widget_loop_parts', 1, 3 );

/**
 * A way to add classes to all widgets with the 'widget' class.
 */
function twenty8teen_widget_params( $params ) {
	$classes = twenty8teen_area_classes( 'widgets', '', false );
	if ( $classes ) {
		if ( preg_match( '/class=([\'"])(.+?)\1/i', $params[0]['before_widget'], $match ) ) {
			$before = explode( ' ', $match[2] );
			if ( in_array( 'widget', $before ) ) {
				$params[0]['before_widget'] = str_ireplace ( $match[2],
					$match[2] . ' ' . $classes, $params[0]['before_widget'] );
			}
		}
	}
	return $params;
}
add_filter( 'dynamic_sidebar_params', 'twenty8teen_widget_params' );

/**
 * For fallback menu, make the parameters match custom menus.
 */
function twenty8teen_page_menu_args( $args ) {
	list( $before, $after ) = explode( '%3$s', $args['items_wrap'] );
	$walker = new Twenty8teen_Walker_Page;
	$wanted = array(
		'sort_column'  => 'menu_order, post_date',
		'menu_id'      => $args['container_id'],
		'menu_class'   => $args['container_class'],
		'before'       => sprintf( $before, esc_attr( $args['menu_id'] ),
			esc_attr( $args['menu_class'] ) ),
		'after'        => $after,
		'show_home'    => false,
		'walker'       => $walker,
	);
	unset( $args['fallback_cb'] );
	return wp_parse_args( $wanted, $args );
}

/**
 * For custom menu, adding an input and label for submenus.
 */
function twenty8teen_nav_menu_start_el( $item_output, $item, $depth, $args ) {
	$classes = empty( $item->classes ) ? array() : (array) $item->classes;
	if ( $classes && in_array( 'menu-item-has-children', $classes ) ||
		in_array( 'page_item_has_children', $classes) ) {
		$item_output .= '<input type="checkbox" id="sub' . $item->ID
			. '"><label for="sub' . $item->ID . '"></label>';
	}
	return $item_output;
}

/**
 * For page menu, adding an input and label for submenus.
 */
function twenty8teen_page_menu_start_el( $item_output, $page, $depth, $args ) {
	if ( isset( $args['pages_with_children'][ $page->ID ] ) ) {
		$item_output .= '<input type="checkbox" id="sub' . $page->ID
			. '"><label for="sub' . $page->ID . '"></label>';
	}
	return $item_output;
}
add_filter( 'walker_page_menu_start_el', 'twenty8teen_page_menu_start_el', 9, 4);

/**
 * In order to filter the page menu, create a new version with a filter.
 */
class Twenty8teen_Walker_Page extends Walker_Page {
	public function start_el( &$output, $page, $depth = 0, $args = array(), $current_page = 0 ) {
		if ( isset( $args['item_spacing'] ) && 'preserve' === $args['item_spacing'] ) {
			$t = "\t";
			$n = "\n";
		} else {
			$t = '';
			$n = '';
		}
		if ( $depth ) {
			$indent = str_repeat( $t, $depth );
		} else {
			$indent = '';
		}

		$css_class = array( 'page_item', 'page-item-' . $page->ID );

		if ( isset( $args['pages_with_children'][ $page->ID ] ) ) {
			$css_class[] = 'page_item_has_children';
		}

		if ( ! empty( $current_page ) ) {
			$_current_page = get_post( $current_page );
			if ( $_current_page && in_array( $page->ID, $_current_page->ancestors ) ) {
				$css_class[] = 'current_page_ancestor';
			}
			if ( $page->ID == $current_page ) {
				$css_class[] = 'current_page_item';
			} elseif ( $_current_page && $page->ID == $_current_page->post_parent ) {
				$css_class[] = 'current_page_parent';
			}
		} elseif ( $page->ID == get_option( 'page_for_posts' ) ) {
			$css_class[] = 'current_page_parent';
		}

		/**
		 * Filters the list of CSS classes to include with each page item in the list.
		 *
		 * @since 2.8.0
		 *
		 * @see wp_list_pages()
		 *
		 * @param array   $css_class    An array of CSS classes to be applied
		 *                              to each list item.
		 * @param WP_Post $page         Page data object.
		 * @param int     $depth        Depth of page, used for padding.
		 * @param array   $args         An array of arguments.
		 * @param int     $current_page ID of the current page.
		 */
		$css_classes = implode( ' ', apply_filters( 'page_css_class', $css_class, $page, $depth, $args, $current_page ) );
		$css_classes = $css_classes ? ' class="' . esc_attr( $css_classes ) . '"' : '';

		if ( '' === $page->post_title ) {
			/* translators: %d: ID of a post */
			$page->post_title = sprintf( __( '#%d (no title)', 'twenty8teen' ), $page->ID );
		}

		$output .= $indent . '<li' . $css_classes . '>';

		$atts = array();
		$atts['href'] = get_permalink( $page->ID );
		$atts = apply_filters( 'page_menu_link_attributes', $atts, $page, $args, $depth );
		$attributes = '';
		foreach ( $atts as $attr => $value ) {
			if ( ! empty( $value ) ) {
				$value = ( 'href' === $attr ) ? esc_url( $value ) : esc_attr( $value );
				$attributes .= ' ' . $attr . '="' . $value . '"';
			}
		}
		/** This filter is documented in wp-includes/post-template.php */
		$title = apply_filters( 'the_title', $page->post_title, $page->ID );

		$args['link_before'] = empty( $args['link_before'] ) ? '' : $args['link_before'];
		$args['link_after'] = empty( $args['link_after'] ) ? '' : $args['link_after'];
		$item_output = '<a' . $attributes . '>';
		$item_output .= $args['link_before'] . $title . $args['link_after'];
		$item_output .= '</a>';

		$output .= apply_filters( 'walker_page_menu_start_el', $item_output, $page, $depth, $args );

		if ( ! empty( $args['show_date'] ) ) {
			if ( 'modified' === $args['show_date'] ) {
				$time = $page->post_modified;
			} else {
				$time = $page->post_date;
			}

			$date_format = empty( $args['date_format'] ) ? '' : $args['date_format'];
			$output .= ' ' . mysql2date( $date_format, $time );
		}
	}

}

/**
 * For admin post editor sidebar, explain featured images.
 */
function twenty8teen_admin_post_thumbnail_html($text) {
	return $text
		. '<p><small>'
		. esc_html__( 'Featured Image will display wherever the widget is placed, using the large image size.', 'twenty8teen' )
		. '</small></p>';
}
add_filter('admin_post_thumbnail_html','twenty8teen_admin_post_thumbnail_html');
